# 
#  ***** BEGIN LICENSE BLOCK *****  
#   
#  Source last modified: $Id: collate.py,v 1.3 2006/04/24 23:34:02 jfinnecy Exp $ 
#   
#  Copyright Notices: 
#   
#  Portions Copyright (c) 1995-2006 RealNetworks, Inc. All Rights Reserved. 
#   
#  Patent Notices: This file may contain technology protected by one or  
#  more of the patents listed at www.helixcommunity.org 
#   
#  1.   The contents of this file, and the files included with this file, 
#  are protected by copyright controlled by RealNetworks and its  
#  licensors, and made available by RealNetworks subject to the current  
#  version of the RealNetworks Public Source License (the "RPSL")  
#  available at  * http://www.helixcommunity.org/content/rpsl unless  
#  you have licensed the file under the current version of the  
#  RealNetworks Community Source License (the "RCSL") available at 
#  http://www.helixcommunity.org/content/rcsl, in which case the RCSL 
#  will apply.  You may also obtain the license terms directly from 
#  RealNetworks.  You may not use this file except in compliance with 
#  the RPSL or, if you have a valid RCSL with RealNetworks applicable 
#  to this file, the RCSL.  Please see the applicable RPSL or RCSL for 
#  the rights, obligations and limitations governing use of the 
#  contents of the file. 
#   
#  2.  Alternatively, the contents of this file may be used under the 
#  terms of the GNU General Public License Version 2 (the 
#  "GPL") in which case the provisions of the GPL are applicable 
#  instead of those above.  Please note that RealNetworks and its  
#  licensors disclaim any implied patent license under the GPL.   
#  If you wish to allow use of your version of this file only under  
#  the terms of the GPL, and not to allow others 
#  to use your version of this file under the terms of either the RPSL 
#  or RCSL, indicate your decision by deleting Paragraph 1 above 
#  and replace them with the notice and other provisions required by 
#  the GPL. If you do not delete Paragraph 1 above, a recipient may 
#  use your version of this file under the terms of any one of the 
#  RPSL, the RCSL or the GPL. 
#   
#  This file is part of the Helix DNA Technology.  RealNetworks is the 
#  developer of the Original Code and owns the copyrights in the 
#  portions it created.   Copying, including reproducing, storing,  
#  adapting or translating, any or all of this material other than  
#  pursuant to the license terms referred to above requires the prior  
#  written consent of RealNetworks and its licensors 
#   
#  This file, and the files included with this file, is distributed 
#  and made available by RealNetworks on an 'AS IS' basis, WITHOUT  
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND REALNETWORKS  
#  AND ITS LICENSORS HEREBY DISCLAIM  ALL SUCH WARRANTIES, INCLUDING  
#  WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS  
#  FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. 
#   
#  Technology Compatibility Kit Test Suite(s) Location:  
#     http://www.helixcommunity.org/content/tck 
#   
# Contributor(s):
# 
# ***** END LICENSE BLOCK *****
# 
"""Base class of build system plugins."""

import sys

class Collate:
    """Base class for preforming a "action" on a module.  For example,
    Collate is subclassed into the Compiler class, which is used to
    run Umake on a module."""

    def __init__(self, module, settings):
        """The arguments are a instance of the Module() class from module.py,
        and the BuildApp.settings hash from buildapp.py."""

        self.module = module
        self.settings = settings
        self.output_func = sys.stdout.write
        self.error_func = sys.stderr.write
        self.debug_func = None
        self.verbose_func = None

    def set_output_func(self, output_func):
        """Set the output callback function.  The callback should take
        a single string as input."""
        self.output_func = output_func
        
    def set_error_func(self, error_func):
        """Same as set_output_func, but for error messages."""        
        self.error_func = error_func

    def set_debug_func(self, output_func):
        """Same as set_output_func, but for debug messages."""
        self.debug_func = output_func
        
    def set_verbose_func(self, error_func):
        """Same as set_output_func, but for verbose messages."""
        self.verbose_func = error_func

    def output(self, text):
        """Output standard messages."""
        if self.output_func:
            self.output_func('%s(%s): %s\n' % (
                self.__class__.__name__,
                self.module.name,
                text))

    def error(self, text):
        """Output an error message."""
        if self.error_func:
            self.error_func('%s(%s) ERROR: %s\n' % (
                self.__class__.__name__,
                self.module.name,
                text))

    def debug(self, text):
        """Output a debug message."""
        if self.debug_func:
            self.debug_func('%s(%s) DEBUG: %s\n' % (
                self.__class__.__name__,
                self.module.name,
                text))

    def verbose(self, text):
        """Output a verbose message."""
        if self.verbose_func:
            self.verbose_func(text)

    def run(self):
        """Override this method in sub-classes to get anything done."""
        pass
