/* ***** BEGIN LICENSE BLOCK *****
 * Source last modified: $Id: pxcontxt.cpp,v 1.1.26.1 2004/07/09 01:54:51 hubbe Exp $
 * 
 * Portions Copyright (c) 1995-2004 RealNetworks, Inc. All Rights Reserved.
 * 
 * The contents of this file, and the files included with this file,
 * are subject to the current version of the RealNetworks Public
 * Source License (the "RPSL") available at
 * http://www.helixcommunity.org/content/rpsl unless you have licensed
 * the file under the current version of the RealNetworks Community
 * Source License (the "RCSL") available at
 * http://www.helixcommunity.org/content/rcsl, in which case the RCSL
 * will apply. You may also obtain the license terms directly from
 * RealNetworks.  You may not use this file except in compliance with
 * the RPSL or, if you have a valid RCSL with RealNetworks applicable
 * to this file, the RCSL.  Please see the applicable RPSL or RCSL for
 * the rights, obligations and limitations governing use of the
 * contents of the file.
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL") in which case the provisions of the GPL are applicable
 * instead of those above. If you wish to allow use of your version of
 * this file only under the terms of the GPL, and not to allow others
 * to use your version of this file under the terms of either the RPSL
 * or RCSL, indicate your decision by deleting the provisions above
 * and replace them with the notice and other provisions required by
 * the GPL. If you do not delete the provisions above, a recipient may
 * use your version of this file under the terms of any one of the
 * RPSL, the RCSL or the GPL.
 * 
 * This file is part of the Helix DNA Technology. RealNetworks is the
 * developer of the Original Code and owns the copyrights in the
 * portions it created.
 * 
 * This file, and the files included with this file, is distributed
 * and made available on an 'AS IS' basis, WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, AND REALNETWORKS HEREBY DISCLAIMS
 * ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 * 
 * Technology Compatibility Kit Test Suite(s) Location:
 *    http://www.helixcommunity.org/content/tck
 * 
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */

// include
#include "hxtypes.h"
#include "hxcom.h"
#include "hxcomm.h"
#include "ihxpckts.h"

// hxcont
#include "hxbuffer.h"

// hxmisc
#include "baseobj.h"

// pxcomlib
#include "pxcontxt.h"

// hxdebug
#include "hxheap.h"
#ifdef _DEBUG
#undef HX_THIS_FILE		
static char HX_THIS_FILE[] = __FILE__;
#endif

PXTestContext::PXTestContext()
{
    m_lRefCount = 0;
}

PXTestContext::~PXTestContext()
{
}

STDMETHODIMP PXTestContext::QueryInterface(REFIID riid, void** ppvObj)
{
    HX_RESULT retVal = HXR_OK;

    if (IsEqualIID(riid, IID_IUnknown))
    {
        AddRef();
        *ppvObj = (IUnknown *) this;
    }
    else if (IsEqualIID(riid, IID_IHXCommonClassFactory))
    {
        AddRef();
        *ppvObj = (IHXCommonClassFactory*) this;
    }
    else
    {
        *ppvObj = NULL;
        retVal  = HXR_NOINTERFACE;
    }

    return retVal;
}

STDMETHODIMP_(UINT32) PXTestContext::AddRef()
{
    return InterlockedIncrement(&m_lRefCount);
}

STDMETHODIMP_(UINT32) PXTestContext::Release()
{
    
    if (InterlockedDecrement(&m_lRefCount) > 0)
        return m_lRefCount;

    delete this;

    return 0;
}

HX_RESULT PXTestContext::CreateObject(PXTestContext** ppObj)
{
    HX_RESULT retVal = HXR_FAIL;

    if (ppObj)
    {
        PXTestContext* pContext = new PXTestContext();
        if (pContext)
        {
            *ppObj = pContext;
            retVal = HXR_OK;
        }
    }

    return retVal;
}

STDMETHODIMP PXTestContext::CreateInstance(REFCLSID rclsid, void** ppUnknown)
{
    HX_RESULT retVal = HXR_OK;

    if (ppUnknown)
    {
        // Set default
        *ppUnknown = NULL;

        if (IsEqualCLSID(rclsid, CLSID_IHXBuffer))
        {
            *ppUnknown = (IUnknown*)(IHXBuffer*)(new CHXBuffer());
            ((IUnknown*)*ppUnknown)->AddRef();
        }
        else
        {
            retVal = HXR_NOINTERFACE;
        }
    }
    else
    {
        retVal = HXR_INVALID_PARAMETER;
    }

    return retVal;

}

STDMETHODIMP PXTestContext::CreateInstanceAggregatable(REFCLSID       rclsid,
                                                       REF(IUnknown*) ppUnknown,
                                                       IUnknown*      pUnkOuter)
{
    return HXR_NOTIMPL;
}

