/* ***** BEGIN LICENSE BLOCK *****
 * Source last modified: $Id: theoravidfmt.h,v 1.5.2.1 2004/11/24 18:11:01 acolwell Exp $
 * 
 * Portions Copyright (c) 1995-2004 RealNetworks, Inc. All Rights Reserved.
 * 
 * The contents of this file, and the files included with this file,
 * are subject to the current version of the RealNetworks Public
 * Source License (the "RPSL") available at
 * http://www.helixcommunity.org/content/rpsl unless you have licensed
 * the file under the current version of the RealNetworks Community
 * Source License (the "RCSL") available at
 * http://www.helixcommunity.org/content/rcsl, in which case the RCSL
 * will apply. You may also obtain the license terms directly from
 * RealNetworks.  You may not use this file except in compliance with
 * the RPSL or, if you have a valid RCSL with RealNetworks applicable
 * to this file, the RCSL.  Please see the applicable RPSL or RCSL for
 * the rights, obligations and limitations governing use of the
 * contents of the file.
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL") in which case the provisions of the GPL are applicable
 * instead of those above. If you wish to allow use of your version of
 * this file only under the terms of the GPL, and not to allow others
 * to use your version of this file under the terms of either the RPSL
 * or RCSL, indicate your decision by deleting the provisions above
 * and replace them with the notice and other provisions required by
 * the GPL. If you do not delete the provisions above, a recipient may
 * use your version of this file under the terms of any one of the
 * RPSL, the RCSL or the GPL.
 * 
 * This file is part of the Helix DNA Technology. RealNetworks is the
 * developer of the Original Code and owns the copyrights in the
 * portions it created.
 * 
 * This file, and the files included with this file, is distributed
 * and made available on an 'AS IS' basis, WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, AND REALNETWORKS HEREBY DISCLAIMS
 * ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 * 
 * Technology Compatibility Kit Test Suite(s) Location:
 *    http://www.helixcommunity.org/content/tck
 * 
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */

#ifndef THEORAVIDFMT_H
#define THEORAVIDFMT_H

#include "vidrendf.h"
#include "vidrend.h"

#include "ogg/ogg.h"

extern "C" {
#include "theora/theora.h"
};

#include "hxslist.h"

typedef struct {
    char* m_pTheoraTag;
    const char* m_pPropName;
} TheoraTagMapping;

class CTheoraVideoFormat : public CVideoFormat
{
public:
    CTheoraVideoFormat(IHXCommonClassFactory* pCCF, 
		       CVideoRenderer* pVidRend,
                       IHXStream* pStream);

    ~CTheoraVideoFormat();

    virtual CMediaPacket* CreateAssembledPacket(IHXPacket* pCodecData);
    virtual CMediaPacket* CreateDecodedPacket(CMediaPacket* pCodedPacket);
    virtual HX_RESULT InitBitmapInfoHeader(HXBitmapInfoHeader &BitmapInfoHeader,
					   CMediaPacket* pVideoPacket);
    virtual void Reset();
    virtual void Close();
    
private:
    typedef enum {stStart,
		  stNeedIdent,
		  stNeedComments,
		  stNeedCodebooks,
		  stReady} State;

    IHXBuffer* OggPktToIHXBuffer(ogg_packet* pOp);
    CMediaPacket* CreateYUVPacket(theora_info* ti, yuv_buffer& yuv, 
                                  UINT32 ulTimestamp);
    IHXBuffer* CreateBuffer(UINT32 ulSize);

    BOOL IsHeader(ogg_packet* pOp) const;
    BOOL IsKeyframe(ogg_packet* pOp) const;

    void HandleIdentHdr(ogg_packet* pOp);
    UINT32 CalcTimestamp(UINT32 ulFrame) const;

    void ChangeState(State newState);
    CMediaPacket* GetMediaPacket(UINT32 uFrameSize,
                                 UINT32 uTimestamp,
                                 HXxSize* pDims);
    void CreatePhaseInfo(UINT32 phaseCt);
    inline UINT32 MulDiv(UINT32 uVal, UINT32 uNum, UINT32 uDen) const;

    void updateBitrateInfo(theora_info* ti);
    void updateTACInfo(theora_comment* tc);

    HX_RESULT getPropName(UINT32 uBaseID, const char* pChildPropName,
                          CHXString& propName);
    HX_RESULT getRegistryID(IUnknown* pUnk, REF(UINT32) uRegID);

    State m_state;
    UINT32 m_ulFPSNum;
    UINT32 m_ulFPSDenom;
    UINT32 m_uAspectNum;
    UINT32 m_uAspectDenom;
    UINT8* m_pPhaseInfo;

    ogg_sync_state m_oy;
    ogg_stream_state m_os;
    ogg_page m_og;
    int m_currentSerialNum;

    theora_state m_ts;
    theora_info m_ti;
    theora_comment m_tc;

    BOOL m_bNeedKeyframe;

    HXxSize m_lastFrameSize;

    IHXStream* m_pStream;
    IHXRegistry* m_pRegistry;

    static const TheoraTagMapping zm_tagInfo[];
};

inline UINT32 
CTheoraVideoFormat::MulDiv(UINT32 uVal, UINT32 uNum, UINT32 uDen) const
{
    UINT32 uQ = uVal / uDen;
    UINT32 uR = uVal - uQ * uDen;

    return uQ * uNum + (uR * uNum) / uDen;
}
#endif /* THEORAVIDFMT_H */
