/* ***** BEGIN LICENSE BLOCK *****
 * Source last modified: $Id: CHXEQProcessor.h,v 1.1.20.3 2004/07/09 01:49:47 hubbe Exp $
 * 
 * Portions Copyright (c) 1995-2004 RealNetworks, Inc. All Rights Reserved.
 * 
 * The contents of this file, and the files included with this file,
 * are subject to the current version of the RealNetworks Public
 * Source License (the "RPSL") available at
 * http://www.helixcommunity.org/content/rpsl unless you have licensed
 * the file under the current version of the RealNetworks Community
 * Source License (the "RCSL") available at
 * http://www.helixcommunity.org/content/rcsl, in which case the RCSL
 * will apply. You may also obtain the license terms directly from
 * RealNetworks.  You may not use this file except in compliance with
 * the RPSL or, if you have a valid RCSL with RealNetworks applicable
 * to this file, the RCSL.  Please see the applicable RPSL or RCSL for
 * the rights, obligations and limitations governing use of the
 * contents of the file.
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL") in which case the provisions of the GPL are applicable
 * instead of those above. If you wish to allow use of your version of
 * this file only under the terms of the GPL, and not to allow others
 * to use your version of this file under the terms of either the RPSL
 * or RCSL, indicate your decision by deleting the provisions above
 * and replace them with the notice and other provisions required by
 * the GPL. If you do not delete the provisions above, a recipient may
 * use your version of this file under the terms of any one of the
 * RPSL, the RCSL or the GPL.
 * 
 * This file is part of the Helix DNA Technology. RealNetworks is the
 * developer of the Original Code and owns the copyrights in the
 * portions it created.
 * 
 * This file, and the files included with this file, is distributed
 * and made available on an 'AS IS' basis, WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, AND REALNETWORKS HEREBY DISCLAIMS
 * ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 * 
 * Technology Compatibility Kit Test Suite(s) Location:
 *    http://www.helixcommunity.org/content/tck
 * 
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */

#ifndef _CHXEQ_PROCESSOR_H_
#define _CHXEQ_PROCESSOR_H_

#include "CHXClientUnknown.h"

#include "enter_hx_headers.h"
#include "hxausvc.h" // IHXAudioHook, IHXAudioPlayer, IHXAudioHookManager, IHXAudioDeviceManager, HXAudioFormat
#include "exit_hx_headers.h"

#include "HXEQEngine.h"

_INTERFACE IHXPlayer;

class CHXEQProcessor : public IHXAudioHook
	// , public IHXSecureStatus // XXXSEH: Add support. Found in common_rn/include/rmadrm.h
{
    DECLARE_UNKNOWN_NOCREATE( CHXEQProcessor )

private:
	
	LONG32 m_lCount;
	IHXAudioPlayer* m_pAudioPlayer;
	EQPTR m_pEQData;
	HXAudioFormat m_AudioFormat;
	bool m_HasHookedAudio;
	bool m_IsEnabled;
	bool m_IsAutoPreGainEnabled;
	int m_PreGain;
	int m_Gains[ EQ_MAXBANDS ];
	int m_RoomSize;
	int m_Reverb;
	
	void DestroyEQData( void );
	bool SetupEQData( void );
	
public:

    virtual ~CHXEQProcessor( void );
    CHXEQProcessor( IHXPlayer* pIHXPlayer );

	void HookAudio( void );
	void UnhookAudio( void );
	
	// IHXAudioHook
	STDMETHOD ( OnInit ) ( THIS_ HXAudioFormat* pFormat );
	STDMETHOD ( OnBuffer ) ( THIS_ HXAudioData* pAudioInData, HXAudioData* pAudioOutData );

	void Enable( bool enable );
	bool IsEnabled( void ) const
	{
		return m_IsEnabled;
	}
	void SetGain( int band, int gain );
	int GetGain( int band ) const;
	void SetPreGain( int preGain );
	int GetPreGain( void ) const;
	void EnableAutoPreGain( bool enableAutoPreGain );
	bool IsAutoPreGainEnabled( void ) const
	{
		return m_IsAutoPreGainEnabled;
	}
	void SetReverb( int roomSize, int reverb );
	void GetReverb( int* pRoomSize, int* pReverb ) const;
};

#endif
