/* ***** BEGIN LICENSE BLOCK *****
 * Source last modified: $Id: rtspprop.h,v 1.1.38.1 2004/07/09 02:05:21 hubbe Exp $
 * 
 * Portions Copyright (c) 1995-2004 RealNetworks, Inc. All Rights Reserved.
 * 
 * The contents of this file, and the files included with this file,
 * are subject to the current version of the RealNetworks Public
 * Source License (the "RPSL") available at
 * http://www.helixcommunity.org/content/rpsl unless you have licensed
 * the file under the current version of the RealNetworks Community
 * Source License (the "RCSL") available at
 * http://www.helixcommunity.org/content/rcsl, in which case the RCSL
 * will apply. You may also obtain the license terms directly from
 * RealNetworks.  You may not use this file except in compliance with
 * the RPSL or, if you have a valid RCSL with RealNetworks applicable
 * to this file, the RCSL.  Please see the applicable RPSL or RCSL for
 * the rights, obligations and limitations governing use of the
 * contents of the file.
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL") in which case the provisions of the GPL are applicable
 * instead of those above. If you wish to allow use of your version of
 * this file only under the terms of the GPL, and not to allow others
 * to use your version of this file under the terms of either the RPSL
 * or RCSL, indicate your decision by deleting the provisions above
 * and replace them with the notice and other provisions required by
 * the GPL. If you do not delete the provisions above, a recipient may
 * use your version of this file under the terms of any one of the
 * RPSL, the RCSL or the GPL.
 * 
 * This file is part of the Helix DNA Technology. RealNetworks is the
 * developer of the Original Code and owns the copyrights in the
 * portions it created.
 * 
 * This file, and the files included with this file, is distributed
 * and made available on an 'AS IS' basis, WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, AND REALNETWORKS HEREBY DISCLAIMS
 * ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 * 
 * Technology Compatibility Kit Test Suite(s) Location:
 *    http://www.helixcommunity.org/content/tck
 * 
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */

#ifndef _RTSPPROP_H_
#define _RTSPPROP_H_

#include "hxstring.h"
#include "hxslist.h"
#include "ihxpckts.h"

class RTSPProperty
{
public:
    enum { PROP_INTEGER, PROP_BUFFER, PROP_STRING };
    RTSPProperty(const char* pName, INT32 propType);
    virtual ~RTSPProperty();

    const char* getName();
    INT32 getType();
    virtual CHXString asString() = 0;

protected:
    char*	m_pName;
    INT32	m_propType;
};

class RTSPIntegerProperty: public RTSPProperty
{
public:
    RTSPIntegerProperty(const char* pName, INT32 value);
    virtual ~RTSPIntegerProperty();

    INT32 getValue();
    virtual CHXString asString() = 0;
protected:
    INT32 m_value;
};

class RTSPBufferProperty: public RTSPProperty
{
public:
    RTSPBufferProperty(const char* pName, IHXBuffer* pValue);
    virtual ~RTSPBufferProperty();

    IHXBuffer* getValue();
    virtual CHXString asString() = 0;
protected:
    IHXBuffer*	m_pValue;
};

class RTSPStringProperty: public RTSPProperty
{
public:
    RTSPStringProperty(const char* pName, const char* pValue);
    virtual ~RTSPStringProperty();

    const char* getValue();
    virtual CHXString asString() = 0;
protected:
    CHXString m_value;
};

class RTSPPropertyList
{
public:
    RTSPPropertyList();
    ~RTSPPropertyList();

    void addProperty(RTSPProperty* pProp);
    RTSPProperty* getProperty(const char* pName);
    RTSPProperty* getFirstProperty();
    RTSPProperty* getNextProperty();

private:
    CHXSimpleList*  m_pPropertyList;
    LISTPOSITION    m_listPos;
};

#endif /* _RTSPPROP_H_ */
